
%*****************************************************
%* author: Jean-Christophe PESQUET                   *
%* institution: LIGM - UMR CNRS 8049         	     *
%*	       Université Paris-Est                      *
%* date: Friday, July, 24th 2009                     *
%*****************************************************


clc
clear all
close all

% fix the blur to noise signal ratio you want to consider
vbsnr=10;%[10,15,20,25,30];%[5,15,25,35];
% fix rho (default value = 10)
vrho=10;%[10,15];
% vectors can be considered ion order to test different rho or BSNR values

for rho=vrho,
    for bsnr=vbsnr,
        randn('state',0)
        rand('state',0)
        
        %image to process
        load -ascii barbara
        x = barbara;
        clear barbara
        %For tests on a smaller image
        %x=x(1:128,1:128);
        
        [N,M] = size(x);
        
        %resolution level
        jm = 4;
        %wavelet choice
        %wavname = 'sym4';
        %mwav='symlets';
        wavname = 'sym8';
        mwav='symlets8';
        qmf = MakeONFilter('Symmlet',8);
        
        %blur size
        L1 = 5;
        L2 = 5;
        %%%%%%%%%%%%%%%%%%%%%%%%%%
        %uniform blur
        h = ones(L1,L2)/(L1*L2);
        H = freqfilt2D(h,N,M);
        %%%%%%%%%%%%%%%%%%%%%%%%%%
        %gaussian blur
        %sigk = 2;
        %h = gaussian_kernel(min(N/2,M/2)-1,sigk);
        %H = freqfilt2D(h,N,M);
        %%%%%%%%%%%%%%%%%%%%%%%%%%
        %for denoising
        %H = ones(N,M);
        %%%%%%%%%%%%%%%%%%%%%%%%%%
        %cosine blur
        %Fc =3/32;
        %H = filtrecos(N,M,Fc);
        
        %image blurring
        xc = real(ifft2(H.*fft2(x)));
        
        %sigma computation to reach fixed bsnr
        sigmab = setbsnr(bsnr,xc);
        
	% parameter for the Wiener-like filter
        etaa = 3;
        
        %when basis union is used (dec is the number of shifts to make)
        %dec = [0 0;1 1];
        %dec = [0 0;1 0];
        %dec = [0 0;0 1;1 1];
        %dec = [0 0];
        %dec = [0 1];
        %dec = [1 0];
        %dec = [1 1];
        dec = [0 0;1 0;0 1;1 1];
        
        dwtmode('per')
        
        
        chi = 3.5;
        alpha = 2.25;
        alphab = 3;
        
        %number of noise realizations
        nitm = 1;
        snr = zeros(2,nitm);
        snrb = zeros(2,nitm);
        snrbt = zeros(2,nitm);
        snrbr = zeros(2,nitm);
        
        for nit = 1:nitm
            fprintf(1,'Noise Realization %d\n',nit);
            r = xc+sigmab*randn(N,M);
            
            mur = mean(r(:));
            
            bsnr = 10*log10(norm(xc,'fro')^2/sigmab^2/N/M);
            snr(:,nit) = calcsnr(r,x);
            message = sprintf('Noise standard deviation = %g SNR = %g PSNR = %g BSNR = %g\n',sigmab,snr(1,nit),snr(2,nit),bsnr);
            disp(message)
            
            %Wiener like filter
            eta = etaa*sigmab^2/(var(r(:))-sigmab^2)
            aH = abs(H)./(abs(H).^2+eta);
            pH = angle(H);
            eH = 1; %deconvolution + denoising
            GGa = aH.^eH.*exp(i*eH*pH);
            GGb = aH.^(1-eH).*exp(-i*(1-eH)*pH);
            
            
            %SURELET with functions taken by Blu and Luisier (F1=Id and F2=exponential)
            %orthogonal bases
            disp('Blu basis')
            [xb,snrb(:,nit)] = decwavsureletbit(r,x,jm,wavname,sigmab,alphab,H,GGa,GGb,eta,rho);
            disp(' ')
            
            %shifted basis union
            disp('Blu basis union')
            [xbt,snrbt(:,nit)] = decwavt2sureletbit(r,x,jm,wavname,dec,sigmab,alphab,H,GGa,GGb,eta,rho);
            disp(' ')
            
            %undecimated wavelets
            disp('Blu redundant')
            [xbr,snrbr(:,nit)] = decwavrsureletbit(r,x,jm,qmf,sigmab,alphab,H,GGa,GGb,eta,rho);
            disp(' ')
            
        end
        
        figure(1)
        imagesc(x)
        colormap(gray(256))
        axis off
        axis('square')
        
        figure(2)
        imagesc(r)
        colormap(gray(256))
        axis off
        axis('square')
        
        figure(3)
        imagesc(xbt)
        colormap(gray(256))
        axis off
        axis('square')
        
        %in last, we take the median value over all the iterations
        snr(:,size(snr,2)+1)=median(snr,2);
        snrb(:,size(snrb,2)+1)=median(snrb,2);
        snrbt(:,size(snrbt,2)+1)=median(snrbt,2);
        snrbr(:,size(snrbr,2)+1)=median(snrbr,2);
        
	% save the results in a .mat file
        tit=['Rest_SURELET','_bnsr=',num2str(bsnr),'_sigmab=',num2str(sigmab),'_rho=',num2str(rho),'_etaa=',num2str(etaa),'_unif5x5.mat'];
        save(tit,'snrbr','snrbt','snrb','snr','sigmab','bsnr','x','r','xb','xbt','xbr');
        clear snr snrst snrbt;
    end
end


